#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#

class Condition:
    def __init__(self):
        return

    def evaluate(self):
        raise NotImplementedError


class UserCondition(Condition):
    def __init__(self,name):
        self.name = name

    def evaluate(self):
        from os import getenv
        return getenv('USER') == self.name


class IssetCondition(Condition):
    def __init__(self,property):
        self.property = property

    def evaluate(self):
        return True


class MultipleCondition(Condition):
    def __init__(self):
        self.operands = []

    def addOperand(self,cond):
        self.operands.append( cond)


class AndCondition(MultipleCondition):
    def __init__(self):
        MultipleCondition.__init__(self)

    def evaluate(self):
        for op in self.operands:
            if op.evaluate() == False:
                return False
        return True
            

class OrCondition(MultipleCondition):
    def __init__(self):
        MultipleCondition.__init__(self)

    def evaluate(self):
        for op in self.operands:
            if (op.evaluate()):
                return True
        return False
            
