#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#
    
#
# HTTP library extension to handle cookies
#

import os
import stat
import urllib
import httplib
import mimetools
import mimetypes

def get_content_type(fd):
    return mimetypes.guess_type(fd.name)[0] or 'application/octet-stream'

class HTTPCookieConnection(httplib.HTTPConnection):
    def __init__(self, host, port=80, debug=False):
        httplib.HTTPConnection.__init__( self, host, port)
        self.cookies = {}
        self.__debug__ = debug
        if self.__debug__:
            self.set_debuglevel(3)

    def __addValue__(self,name,value):
        self.__body__ += '--%s\r\n' % self.__boundary__
        self.__body__ += 'Content-Disposition: form-data; name="%s"\r\n' % name
        self.__body__ += '\r\n'
        self.__body__ += '%s\r\n' % value

    def __addFile__(self,name,filename):
        fd = open(filename,'r')
        self.__body__ += '--%s\r\n' % self.__boundary__
        self.__body__ += 'Content-Disposition: form-data; name="%s"; filename="%s"\r\n' % (name, filename)
        self.__body__ += 'Content-Type: %s\r\n' % get_content_type(fd)
        file_size = os.fstat(fd.fileno())[stat.ST_SIZE]
        self.__body__ += 'Content-Length: %d\r\n' % file_size
        self.__body__ += '\r\n'
        while True:
            data = fd.read()
            if not data:
                break
            self.__body__ += data.decode('iso8859-15')
        fd.close()
        self.__body__ +='\r\n'

    def __extract_cookies__( self, response):
        headers = response.msg.getheaders('set-cookie')
        for header in headers:
            if self.__debug__:
                print '-> cookie header ' + str(header)
            s = header.split(';')[0]
            i = s.find('=')
            name = s[0:i]
            value = s[i+1:]
            if self.__debug__:
                print '-> got cookie ' + name + '=' + value
            self.cookies[name] = value

    def __add_cookies__(self):
        if len(self.cookies) != 0:
            if self.__debug__:
                print '-> cookies =', self.cookies
            c = ''
            first = True
            for n,v in self.cookies.iteritems():
                if not first:
                    c += '; '
                c = n + '=' + v
                first = False
            self.__headers__[ 'Cookie' ] = c

    def __build_post_body__(self,values,filenames):
        if filenames is None:
            self.__headers__[ 'Content-Type' ] = 'application/x-www-form-urlencoded'
            self.__body__ = urllib.urlencode(values)
        else:
            self.__boundary__ = mimetools.choose_boundary()
            self.__headers__[ 'Content-Type' ] = 'multipart/form-data; boundary=%s' % self.__boundary__ 
            self.__body__ = ''
            for k,v in values.iteritems():
                self.__addValue__(k,v)
            for k,v in filenames.iteritems():
                self.__addFile__(k,v)
            self.__body__ += '--%s--\r\n' % self.__boundary__
            self.__body__ += '\r\n'

    def post( self, url, values = None, filenames = None):
        self.__headers__ = {}
        self.__add_cookies__()
        self.__build_post_body__(values,filenames)

        print 'sending POST request to http://%s:%d%s' % ( self.host, self.port, url)
        if self.__debug__:
            print '-> headers=' + str(self.__headers__)
            print '-> body=' + self.__body__
        
        self.connect()
        self.request( 'POST', url, self.__body__.encode('iso8859-15'), self.__headers__)
        response = self.getresponse()
        self.__extract_cookies__( response)

        data = response.read()
        self.close()

        return (response,data)

    def get( self, url):
        self.__headers__ = {}
        self.__add_cookies__()
        print '-> sending GET request to http://%s:%d%s' % ( self.host, self.port, url)
        
        self.request( 'GET', url, headers = self.__headers__)

        response = self.getresponse()
        self.__extract_cookies__( response)

        data = response.read()
        self.close()

        return (response,data)


# 
# Test code
#
if __name__ == '__main__':
    import random
    import datetime

    c = HTTPCookieConnection( 'stalker', 8080)
    r1 = c.post( '/xwiki/bin/loginsubmit/XWiki/XWikiLogin', { 'j_username' : 'fdechelle', 'j_password' : 'fdechelle', 'j_rememberme' : 'true'} )
    r2 = c.post( '/xwiki/bin/save/QA/TestCookie', { 'content' : "1 Test cookie\n\nPour tester les cookies depuis python\n\n1.1 Ajoute par python le %s\n\nLa valeur de i est : %d\n\n" % ( str(datetime.datetime.today()), random.randint(1,100)) } )
    r3 = c.post( '/xwiki/bin/upload/QA/TestCookie', values = { 'filename' : "file_%d.jpg" % (random.randint(1,10),)}, filenames = { 'filepath' : '/home/fdechelle/Desktop/86202607_53d78f048c.jpg'} )

