#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#

import sys
import imp

def find_package_implementation(name):
    try:
        t = imp.find_module(name)
        return imp.load_module(name,t[0],t[1],t[2])
    except:
        return None

class PackageImplementation(object):
    @classmethod
    def getImplementation(cls):
        if hasattr(cls, '__implementation__'):
            return cls.__implementation__
        for i in [('rpm',RpmPackageImplementation),('deb',DebianPackageImplementation)]:
            module = find_package_implementation(i[0])
            if module is None:
                continue
            cls.__implementation__ = i[1](module)
            if not cls.__implementation__ is None:
                return cls.__implementation__
        cls.__implementation__ = DummyPackageImplementation()
        return cls.__implementation__

    def check(self):
        raise NotImplementedError

def isDebian():
    return isinstance(PackageImplementation.getImplementation(),DebianPackageImplementation)

def isRpm():
    return isinstance(PackageImplementation.getImplementation(),RpmPackageImplementation)

class DummyPackageImplementation(PackageImplementation):
    def check(self,name):
        return True

class RpmPackageImplementation(PackageImplementation):
    def __init__(self,module):
        self.__module__ = module

    def check(self,name):
        ts = self.__module__.TransactionSet()
        mi = ts.dbMatch( 'name', name)
        for hdr in mi:
            if hdr[self.__module__.RPMTAG_NAME] == name:
                return True
        return False

class DebianPackageImplementation(PackageImplementation):
    def __init__(self,module):
        self.__module__ = module

    def check(self):
        return True

class Package:
    def __init__(self,name):
        self.name = name

    def check(self):
        return PackageImplementation.getImplementation().check( self.name)

if __name__ == '__main__':
    print 'Checking for package: ' + sys.argv[1]
    p = Package( sys.argv[1])
    if p.check():
        print 'installed'
    else:
        print 'not installed'

