#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#

import os
import threading
import subprocess
import shutil
import filecmp
import re
import datetime
from edostestrunner.config import Config
from edostestrunner.package import isDebian, isRpm
from edostestrunner.reporting import Reporting

class ConfigurationError:
    """Raised when reading platform configuration failed"""
    pass    

class HardwareConfigurationError:
    """Raised when reading hardware platform configuration failed"""
    pass    

class SoftwareConfigurationError:
    """Raised when reading software platform configuration failed"""
    pass    


class Configuration:
    def __init__(self,stateFileName,command):

        platformstatedir = Config.getConfig().get( 'Global', 'platformstatedir')

        if not os.path.exists( platformstatedir):
            os.makedirs( platformstatedir)

        self.stateFileName = platformstatedir + '/' + stateFileName
        previousStateFileName = self.stateFileName + '.previous'

        # Move previous state file
        try:
            shutil.move( self.stateFileName, previousStateFileName)
        except:
            1

        # Launch the command
        p = subprocess.Popen( command, shell=True, stdin=None, stdout=open(self.stateFileName,'w'), stderr=subprocess.STDOUT, close_fds=True)
        returncode = p.wait()

        if returncode != 0:
            raise ConfigurationError()

        # Compare new state file with previous state file
        if os.path.exists( previousStateFileName) and filecmp.cmp( self.stateFileName, previousStateFileName):
            self.hasChanged = False
        else:
            self.hasChanged = True

    def getConfigAsText(self):
        fd = open( self.stateFileName, 'r')
        return fd.read()


class HardwareConfiguration(Configuration):
    def __init__(self):
        Configuration.__init__( self, 'hwconfig.xml', '/usr/sbin/lshw -xml')


class SoftwareConfiguration(Configuration):
    def __init__(self):
        if isDebian():
            cmd = '/usr/bin/dpkg -l'
        elif isRpm():
            cmd = '/bin/rpm -qa'
        else:
            cmd = 'echo None'
        Configuration.__init__( self, 'swconfig.txt', cmd)


class Platform:
    @classmethod
    def getPlatform(cls):
        if not hasattr(cls, '__platform__'):
            cls.__platform__ = Platform()
        return cls.__platform__

    def __init__(self):
        self.identification = self.__identify__()
#        self.hardwareConfiguration = HardwareConfiguration()
#        self.softwareConfiguration = SoftwareConfiguration()
#        if self.hasChanged():
#            self.reportPlatform()

    def __identify__(self):
        command = "/sbin/ifconfig eth0 | awk '/eth0/ {print $5}'"
        p = subprocess.Popen( command, shell=True, stdin=None, stdout=subprocess.PIPE, stderr=None, close_fds=True)
        returncode = p.wait()
        out = p.stdout.read()
        #out = re.sub( ":", "_", out)
        out = re.sub( "\n", "", out)
        return out

    def hasChanged(self):
        return self.hardwareConfiguration.hasChanged or self.softwareConfiguration.hasChanged

    def reportPlatform( self):
        pass
#        Reporting.getReporter().reportPlatform( 
#            owner_username = Config.getConfig().get('Identification','username'),
#            identification = self.identification,
#            state_date = str(datetime.datetime.utcnow()),
#            software_config = self.softwareConfiguration.getConfigAsText(),
#            hardware_config = self.hardwareConfiguration.getConfigAsText())



# ########################################
# test code

def test(config,name):
    if config.hasChanged:
        print name + ' configuration has changed'
    else:
        print name + ' configuration has not changed'
#    print config.getConfigAsText()
    
if __name__ == '__main__':
    test( PlatformHardwareConfiguration(), 'Hardware')
    test( PlatformSoftwareConfiguration(), 'Software')
