#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#

from xml.dom.ext.reader import PyExpat
from xml import xpath
import xml.dom.ext
import os
import sys
import subprocess
import datetime
from edostestrunner.package import Package
from edostestrunner.condition import AndCondition,OrCondition,UserCondition,IssetCondition
from edostestrunner.command import ShellCommand
from edostestrunner.testreporter import NullReporter
from edostestrunner.testreporter import makeTestReportId
#from edostestrunner.platform import Platform

class Test:
    def __init__( self, xml_file):
        reader = PyExpat.Reader()
        self.domDocument = reader.fromStream( open(xml_file))
        self.__buildDescription__()
        self.__buildPackage__()
        self.__buildCondition__()
        self.__buildTest__()

    def __buildDescription__( self):
        node = xpath.Evaluate( '/test/description', self.domDocument)
        self.name = node[0].attributes[(None,'name')].value
        self.suite = node[0].attributes[(None,'suite')].value
        self.description = node[0].childNodes[0].nodeValue

    def __buildPackage__( self):
        node = xpath.Evaluate( '/test/package', self.domDocument)
        name = node[0].attributes[(None,'name')].value
        self.package = Package( name.encode('ascii'))

    def __buildCondition__( self):
        cond = xpath.Evaluate( '/test/condition/*', self.domDocument)
        self.condition = self.__buildConditionAux__( cond[0])

    def __buildConditionAux__( self,node):
        if (node == None):
            return None
        if node.nodeName == 'user':
            return UserCondition( node.attributes[(None,'name')].value)
        if node.nodeName == 'isset':
            return IssetCondition( node.attributes[(None,'property')].value)
        if node.nodeName == 'and':
            c = AndCondition()
        elif node.nodeName == 'or':
            c = OrCondition()
        for childNode in xpath.Evaluate( '*', node):
            c.addOperand( self.__buildConditionAux__(childNode))
        return c

    def __buildArgList__( self, node, argList):
        for argNode in xpath.Evaluate( './arg', node):
            argList.append( argNode.attributes[(None,'value')].value )

    def __buildTest__( self):
        self.tests = []
        for node in xpath.Evaluate( '/test/test-runner/*', self.domDocument):
            if node.nodeName == 'shell':
                script = node.attributes[(None,'script')].value
                path = None
                if (None,'path') in node.attributes:
                    path = node.attributes[(None,'path')].value
                    script = path + '/' + script

                ignoreReturnCode = False
                if (None,'ignoreReturnCode') in node.attributes and node.attributes[(None,'ignoreReturnCode')].value == 'true':
                    ignoreReturnCode = True

                shortLog = False
                if (None,'shortLog') in node.attributes and node.attributes[(None,'shortLog')].value == 'true':
                    shortLog = True
                
                shellTest = ShellCommand( script, ignoreReturnCode, shortLog)
                self.__buildArgList__( node, shellTest.args)
                self.tests.append( shellTest)

    def checkPackage( self):
        if not self.package.check():
            self.result = 'UNSUPPORTED'
            print 'Error: package %s is not installed' % self.package.name
            sys.exit(1)

    def checkCondition( self):
        if not self.condition.evaluate():
            self.result = 'UNSUPPORTED'
            print 'Error: test pre-condition evaluates to false'
            sys.exit(1)

    def run( self, testReporter = NullReporter(), testDirectory = '/var/tmp/edos-tests', tester= 'NoName'):
        self.checkPackage()
        
        self.checkCondition()

        self.testReportId = makeTestReportId(self.name,self.suite)

#        Platform.getPlatform().reportPlatform()

        if not os.path.exists( testDirectory):
            os.makedirs( testDirectory)

        basedir = '%s-%d' % (self.name, os.getpid())

        testReporter.reportRunning(
            test_report_id = self.testReportId,
            test_suite_name = self.suite,
            test_name = self.name,
            platform_identification = 'None',
            tester_username = tester,
            status = 'RUNNING',
            launch_date = str(datetime.datetime.utcnow()))

        count = 0
        for test in self.tests:
            test.run('%s/%s/%03d' % (testDirectory, basedir, count))
            count += 1

        result = 'PASS'
        for test in self.tests:
            if not test.ignoreReturnCode and test.returncode != 0:
                result = 'FAIL'
                break

        self.shortLog = u''
        for test in self.tests:
            if hasattr(test, 'shortLog'):
                self.shortLog += test.shortLog

        self.zipFile = '%s.zip' % basedir
        self.zipFileFullPath = testDirectory + '/' + self.zipFile
        p = subprocess.Popen( 'zip -r %s %s' % (self.zipFile, basedir),
                              shell=True,
                              stdin=None,
                              stdout=None,
                              stderr=None,
                              close_fds=True,
                              cwd=testDirectory)
        p.wait()

        testReporter.reportCompleted(
            test_report_id = self.testReportId,
            status = 'COMPLETED',
            result = result,
            short_log = self.shortLog,
            short_log_format = 'RAW',
            log_zip_file = self.zipFileFullPath,
            completion_date = str(datetime.datetime.utcnow()))
        


