#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#
#
# Xwiki test reporter
#

import os.path
from edostestrunner.testreporter import TestReporter
from edostestrunner.httplibcookie import HTTPCookieConnection

def explodeDate(d,name):
    for k in ['year','month','day','hour','minute','second','microsecond']:
        d.update({name+'_'+k:getattr(d[name],k)})

class XwikiTestReporter(TestReporter):
    """ Reports a test using a HTTP POST request to a Xwiki server
    """

    def __init__(self, host = 'mandriva.edos-project.org', port = 80, test_report_service_url = '/xwiki/bin/view/QA/TestReportService', platform_service_url = '/xwiki/bin/view/QA/PlatformService', login_url = '/xwiki/bin/loginsubmit/XWiki/XWikiLogin', username = 'guest', password = 'guest'):
        self.host = host
        self.port = int(port)
        self.connection = HTTPCookieConnection( self.host, self.port, debug=True)
        self.testReportServiceUrl = test_report_service_url
        self.platformServiceUrl = platform_service_url
        self.loginUrl = login_url
        self.username = username
        self.password = password
        self.loggedIn = False

    def __str__(self):
        return "XwikiTestReporter( host=%s, port=%d, username=%s, password=%s)" % (self.host, self.port, self.username, self.password)

    def loginIfNeeded(self):
        if self.loggedIn:
            return
        self.connection.post( self.loginUrl, { 'j_username' : self.username, 'j_password' : self.password, 'j_rememberme' : 'true'} )
        
    def reportPlatform(self, **args):
        args.update( raw = 1, xpage = 'plain')
        #explodeDate( args,'state_date')
        self.loginIfNeeded()
        self.connection.post( self.platformServiceUrl, args)

    def reportRunning(self, **args):
        # raw = 0 for XWiki 1.0-beta-3
        args.update( request_type = 'running', raw = '0', xpage = 'plain')
        #explodeDate( args,'launch_date')
        self.loginIfNeeded()
        self.connection.post( self.testReportServiceUrl, args)

    def reportCompleted(self, **args):
        # raw = 0 for XWiki 1.0-beta-3
        args.update( request_type = 'completed', raw = '0', xpage = 'plain')
        #explodeDate( args,'completion_date')
        temp = args.copy()
        del temp['log_zip_file']
        self.loginIfNeeded()
        self.connection.post( self.testReportServiceUrl, temp)
        # FIXME: add parameter for upload base url
        self.connection.post( '/xwiki/bin/upload/TestReport/' + str(args['test_report_id']), values = { 'filename' : os.path.basename( args['log_zip_file'])}, filenames = { 'filepath' : args['log_zip_file'] })
